@@ -1,14 +1,12 @@
+#define _BSD_SOURCE
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
-#define __BSD_SOURCE
-#define __FAVOR_BSD
 #include <netinet/in_systm.h>
 #include <netinet/in.h>
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
-#define _SCANLOGD_NETINET
 
 #include "params.h"
 #include "in.h"
