@@ -1,4 +1,6 @@
+#define _BSD_SOURCE
 #include <stdio.h>
+
 #include <pcap.h>
 
 #include "params.h"
@@ -27,7 +29,7 @@ int in_init(void)
 		return 1;
 	}
 
-	if (pcap_compile(p, &filter, "tcp", 1, 0)) {
+	if (pcap_compile(p, &filter, SCANLOGD_PCAP_FILTER, 1, 0)) {
 		pcap_perror(p, "pcap_compile");
 		return 1;
 	}
