@@ -1,5 +1,5 @@
 /*
- * scanlogd v2.2 by Solar Designer <solar@false.com>.
+ * scanlogd by Solar Designer <solar at openwall.com>.
  *
  * You're allowed to do whatever you like with this software (including
  * re-distribution in any form, with or without modification), provided
@@ -9,7 +9,10 @@
  * There's absolutely no warranty.
  */
 
+#define _BSD_SOURCE
 #include <stdio.h>
+#include <fcntl.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <signal.h>
 #include <string.h>
@@ -21,22 +24,15 @@
 #include <syslog.h>
 #include <sys/times.h>
 #include <sys/types.h>
-#define __BSD_SOURCE
-#define __FAVOR_BSD
 #include <netinet/in_systm.h>
 #include <netinet/in.h>
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
-#define _SCANLOGD_NETINET
 #include <arpa/inet.h>
 
 #include "params.h"
 #include "in.h"
 
-#ifdef USE_RLOG
-#include "rlog.h"
-#endif
-
 #define HF_DADDR_CHANGING		0x01
 #define HF_SPORT_CHANGING		0x02
 #define HF_TOS_CHANGING			0x04
@@ -170,12 +166,6 @@ prepare:
 	syslog(SYSLOG_LEVEL,
 		"%s to %s..., %s%s%s @%s",
 		s_saddr, s_daddr, s_flags, s_tos, s_ttl, s_time);
-
-#ifdef USE_RLOG
-	rlog(
-		"%s to %s..., %s%s%s @%s",
-		s_saddr, s_daddr, s_flags, s_tos, s_ttl, s_time);
-#endif
 }
 
 /*
@@ -191,14 +181,10 @@ static void safe_log(struct host *info)
 	if (now - last > LOG_DELAY_THRESHOLD || now < last) count = 0;
 	if (++count <= LOG_COUNT_THRESHOLD + 1) last = now;
 
-	if (count <= LOG_COUNT_THRESHOLD) {
+	if (count <= LOG_COUNT_THRESHOLD)
 		do_log(info);
-	} else if (count == LOG_COUNT_THRESHOLD + 1) {
+	else if (count == LOG_COUNT_THRESHOLD + 1)
 		syslog(SYSLOG_LEVEL, "More possible port scans follow");
-#ifdef USE_RLOG
-		rlog("More possible port scans follow");
-#endif
-	}
 }
 
 /*
@@ -393,16 +379,18 @@ static void drop_root(void)
 		exit(1);
 	}
 
+#ifdef SCANLOGD_CHROOT
+	if (chroot(SCANLOGD_CHROOT)) return pexit("chroot");
+	if (chdir("/")) return pexit("chdir");
+#endif
+
 	groups[0] = groups[1] = pw->pw_gid;
 	if (setgroups(1, groups)) pexit("setgroups");
 	if (setgid(pw->pw_gid)) pexit("setgid");
 	if (setuid(pw->pw_uid)) pexit("setuid");
 }
-#else
-static void cleanup(int signum)
-{
-	exit(0);	/* Just so that atexit(3) jobs are called */
-}
+#elif defined(SCANLOGD_CHROOT)
+#warning SCANLOGD_CHROOT makes no sense without SCANLOGD_USER; ignored.
 #endif
 
 /*
@@ -410,28 +398,19 @@ static void cleanup(int signum)
  */
 int main(void)
 {
-#ifndef SCANLOGD_USER
-/* We probably can't disable promiscuous mode if we aren't running as root */
-	signal(SIGTERM, cleanup);
-	signal(SIGINT, cleanup);
-	signal(SIGHUP, cleanup);
-#endif
+	int dev_null_fd;
 
 /* Initialize the packet capture interface */
 	if (in_init()) return 1;
 
+/* Prepare for daemonizing */
 	chdir("/");
 	setsid();
 
-#ifdef USE_RLOG
-	errno = 0;
-	if (openrlog(RLOG_ID)) {
-		fprintf(stderr,
-			"openrlog(\"" RLOG_ID "\"): %s\n",
-			errno ? strerror(errno) : "Failed");
-		return 1;
-	}
-#endif
+/* Must do these before chroot'ing */
+	tzset();
+	openlog(SYSLOG_IDENT, LOG_NDELAY, SYSLOG_FACILITY);
+	dev_null_fd = open("/dev/null", O_RDONLY);
 
 /* We can drop root now */
 #ifdef SCANLOGD_USER
@@ -455,13 +434,19 @@ int main(void)
 
 	setsid();
 
+/* Just assume that stdin, stdout, and stderr fd's were open at startup and
+ * thus are indeed not allocated to anything else. */
+	if (dev_null_fd >= 0) {
+		dup2(dev_null_fd, STDIN_FILENO);
+		dup2(dev_null_fd, STDOUT_FILENO);
+		dup2(dev_null_fd, STDERR_FILENO);
+		if (dev_null_fd >= 3) close(dev_null_fd);
+	}
+
 /* Initialize the state. All source IP addresses are set to 0.0.0.0, which
  * means the list entries aren't in use yet. */
 	memset(&state, 0, sizeof(state));
 
-/* Huh? */
-	openlog(SYSLOG_IDENT, 0, SYSLOG_FACILITY);
-
 /* Let's start */
 	in_run(process_packet);
 
